// -*-c++-*-
// Copyright (C) 1995 NEC Corporation, Tokyo, Japan.

#include <Hs/HsAVdesc.h>
#include <Hs/HsDigitizedVideo.h>
#include <Hs/HsDefaultColormap.h>
#include <Hs/HsEffect.h>
#include <Hs/HsMPEGVideo.h>
#include <Hs/HsJPEGVideo.h>
#include <Hs/HsTitle.h>
#include <Hs/HsViewer.h>

#include "StringChange.h"
#include "EfPlayer.h"

#include <InterViews/action.h>
#include <InterViews/bitmap.h>
#include <InterViews/display.h>
#include <InterViews/font.h>
#include <InterViews/input.h>
#include <InterViews/layout.h>
#include <InterViews/polyglyph.h>
#include <InterViews/patch.h>
#include <InterViews/stencil.h>
#include <InterViews/style.h>
#include <InterViews/telltale.h>

#include <IV-look/kit.h>
#include <IV-look/dialogs.h>
#include <IV-look/button.h>

#include <IV-X11/xdisplay.h>

#include <OS/string.h>
#include <sys/time.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define AVOUTFILE "effector.av"

#ifdef nec_ews
extern "C" {
	int gettimeofday( struct timeval* );
};
#endif

extern DataType type_check( const char* name );

declareActionCallback(EffectPlayer)
implementActionCallback(EffectPlayer)

declareFieldEditorCallback(EffectPlayer)
implementFieldEditorCallback(EffectPlayer)

declareFileChooserCallback(EffectPlayer)
implementFileChooserCallback(EffectPlayer)

struct EffectPlayerInfo
{
   const char* str;
   ActionMemberFunction(EffectPlayer) func;
};

EffectPlayerInfo avmode1[] = {
   { " Wipe         ", &EffectPlayer::update},
   { " Slide        ", &EffectPlayer::update},
   { " Iris Square  ", &EffectPlayer::update},
   { " Iris Round   ", &EffectPlayer::update},
   { " Linear Boxes ", &EffectPlayer::update},
   { " Spiral Boxes ", &EffectPlayer::update},
   { " Dissolve     ", &EffectPlayer::update},
   { " Zoom         ", &EffectPlayer::update},
   { " Stretch      ", &EffectPlayer::update},
   { " No Effect    ", &EffectPlayer::update},
   { nil }
};

EffectPlayerInfo avmode2[] = {
   { "  Reverse    ", &EffectPlayer::update},
   { "  Dimension  ", &EffectPlayer::update},
   { nil }
};

EffectPlayerInfo avmode3[] = {
   { "media1", &EffectPlayer::set_name},
   { "media2", &EffectPlayer::set_name},
   { nil }
};

EffectPlayer::EffectPlayer():Glyph()
{
  viewer_      = nil;
  window_      = nil;
  filename1_   = nil;
  filename2_   = nil;
  avfile_      = nil;
  oldnamelist_[0] = nil;
  newnamelist_[0] = nil;
  id_    = 0;
  idcnt_ = 0;
  body_ = make();
  Resource::ref(body_);
}

void EffectPlayer::request(Requisition& requisition) const
{
  if(body_ != nil){
     body_->request(requisition);
  }
}

void EffectPlayer::allocate(Canvas* c, const Allocation& a, Extension& ext)
{
  if(body_ != nil){
    body_->allocate(c, a, ext);
  }
}

void EffectPlayer::draw(Canvas* canvas, const Allocation& a) const
{
  if(body_ != nil)
    body_->draw(canvas, a);
}

void EffectPlayer::pick(Canvas* c, const Allocation& a, int depth, Hit& h)
{
   if (body_ != nil) {
      body_->pick(c, a, depth, h);
   } else {
      Glyph::pick(c, a, depth, h);
   }        
}

Glyph* EffectPlayer::make()
{
   WidgetKit& kit    = *WidgetKit::instance();
   LayoutKit& layout = *LayoutKit::instance();
   DialogKit& dialog = *DialogKit::instance();

   Action* act1 =
      new ActionCallback(EffectPlayer)(this, &EffectPlayer::change_media);
   Action* act2 =
      new ActionCallback(EffectPlayer)(this, &EffectPlayer::map);
   Action* act3 =
      new ActionCallback(EffectPlayer)(this, &EffectPlayer::avdump);
   Action* act4 =
      new ActionCallback(EffectPlayer)(this, &EffectPlayer::quit);

   int i = 0;
   TelltaleGroup* group1 = new TelltaleGroup;   
   Glyph* box1 = layout.vbox( 24 );
   while( avmode1[i].str != nil ){
     Action* a1 = new ActionCallback(EffectPlayer)(this, avmode1[i].func);
     Glyph* g1  = layout.hbox( kit.label( avmode1[i].str ), layout.hglue() );
     buttons1_[i] = kit.radio_button( group1, g1, a1 );
     box1->append(buttons1_[i]);
     i++;
   }
   buttons1_[i] = nil;

   i = 0;
   Glyph* box2 = layout.hbox( 2 );
   while( avmode2[i].str != nil ){
     Action* a2 = new ActionCallback(EffectPlayer)(this, avmode2[i].func);
     buttons2_[i] = kit.palette_button( avmode2[i].str, a2 );
     box2->append(buttons2_[i]);
     i++;
   }
   buttons2_[i] = nil;
   i = 0;
   Glyph* box3 = layout.vbox( 2 );
   TelltaleGroup* group2 = new TelltaleGroup;   
   while( avmode3[i].str != nil ){
     Action* a3 = new ActionCallback(EffectPlayer)(this, avmode3[i].func);
     buttons3_[i] = kit.radio_button( group2, avmode3[i].str, a3 );
     box3->append(buttons3_[i]);
     i++;
   }

   char* pathname = getenv("DATA_DIR");
   if(pathname){
     nfield1_ = dialog.field_editor( pathname, kit.style(), nil );
     nfield2_ = dialog.field_editor( pathname, kit.style(), nil );
   }
   else{
     nfield1_ = dialog.field_editor( ".", kit.style(), nil );
     nfield2_ = dialog.field_editor( ".", kit.style(), nil );
   }
   
   
   InputHandler* namehandler = 
     new InputHandler(
	     layout.hbox( 
	        layout.vbox( layout.vglue(), buttons3_[0],
			     layout.vglue(), buttons3_[1], layout.vglue() ),
		layout.hspace(3),
	        layout.vbox( nfield1_ , nfield2_ )
			 ),
		      kit.style() );
   namehandler->append_input_handler( nfield1_ );
   namehandler->append_input_handler( nfield2_ );


   i = 0;
   mdiffer_ = 
     dialog.field_editor( "0", kit.style(),
			  new FieldEditorCallback(EffectPlayer)( this, &EffectPlayer::accept3, &EffectPlayer::cancel3 ) );

   mstart_ = 
     dialog.field_editor( "0", kit.style(),
			  new FieldEditorCallback(EffectPlayer)( this, &EffectPlayer::accept1, &EffectPlayer::cancel1 ) );

   mend_ = 
     dialog.field_editor( "0", kit.style(),
			  new FieldEditorCallback(EffectPlayer)( this, &EffectPlayer::accept2, &EffectPlayer::cancel2 ) );


   Glyph* mbox = layout.vbox( box1, box2 );

   InputHandler* handler = 
     new InputHandler(
	     layout.vbox( 
	        layout.hbox( layout.hspace(5),
			     kit.label("difference "), mdiffer_ ),
	        layout.hbox( layout.hspace(5),
			     kit.label("start      "), mstart_  ),
	        layout.hbox( layout.hspace(5),
			     kit.label("end        "), mend_    )
			 ),
                       kit.style() );
   handler->append_input_handler( mdiffer_ );
   handler->append_input_handler( mstart_  );
   handler->append_input_handler( mend_    );


   
   if(pathname)
     fchooser_ =
       dialog.file_chooser(pathname, kit.style(), 
			   new FileChooserCallback(EffectPlayer)(this,&EffectPlayer::choose));
   else
     fchooser_ =
       dialog.file_chooser( ".", kit.style(),
			   new FileChooserCallback(EffectPlayer)(this,&EffectPlayer::choose));

   Resource::ref(fchooser_);
   
   duration1_ = new StringChange( "   0  ", kit.font(), kit.foreground() );
   duration2_ = new StringChange( "   0  ", kit.font(), kit.foreground() );
   duration3_ = new StringChange( "   0  ", kit.font(), kit.foreground() );

   Glyph* dbox = 
     layout.vbox( 
	layout.hbox( kit.label(" effect:"),duration1_,kit.label(" frame")),
	layout.hbox( kit.label(" media1:"),duration2_,kit.label(" frame")),
	layout.hbox( kit.label(" media2:"),duration3_,kit.label(" frame"))
		 );

   Style* s;
   kit.push_style( new Style( kit.style() ) );
   s = kit.style();
   s->attribute("*PushButton*minimumWidth", "2");
   Glyph* buttons = 
     kit.outset_frame( 
	     layout.vbox(
	      layout.margin(
	           layout.hbox( 
			        layout.hglue(), kit.push_button("Play",  act1),
				layout.hglue(), kit.push_button("CreateAV",
                                                                 act3),
			        layout.hglue(), kit.push_button("Quit",  act4),
			        layout.hglue()
			       ), 3
			    )
			 )
		      );
   kit.pop_style();

   kit.push_style( new Style( kit.style() ) );
   s = kit.style();
   s->attribute( "font", "*-courier-medium-r-normal--18-180-75-75-*" );
   box_ = layout.vbox();
   box_->append( layout.hspace(kit.font()->width(' ')*20) );
   box_->append( layout.vglue() );
   patch_ = new Patch( box_ );
   kit.pop_style();

   Glyph* mlabel = layout.vmargin( kit.label("Messages:"), 3 );


   Glyph* body = 
      layout.vbox(
	  layout.hbox( make_margin( kit.outset_frame( namehandler ) ) ),
	  layout.hbox(
	     make_margin( layout.vbox( fchooser_, layout.hspace(200) ) ),
	     make_margin(
                layout.vbox(kit.outset_frame( mbox ),
			    kit.outset_frame(layout.hbox(dbox,
							 layout.hglue() ) ),
			    kit.outset_frame( handler ),
			    buttons
			    )
			 ),
	     make_margin( 
                kit.outset_frame( 
                           layout.margin( layout.vbox( mlabel, patch_ ), 3 )
				 )
			 )
		      )		 
		  );

   
   return body;
}

HsViewer* EffectPlayer::view()
{
   return viewer_;
}

void EffectPlayer::map()
{
   if(!window_)
     return;

   if(window_->is_mapped()){
      viewer_->stop();
      if(window_ != nil){
	 HsDefaultColormap::uninstall(window_);
	 window_->unmap();
      }
   } else{
      Glyph* g = window_->glyph();
      Resource::ref(g);
      delete window_;
      window_ = new TransientWindow( g );
      window_->map();
      show_message("Effect Playing !!");
      HsDefaultColormap::install(window_);
   }
}

void EffectPlayer::quit()
{
   if(viewer_)
      viewer_->stop();
   HsSession::instance()->quit();
}

void EffectPlayer::update()
{
  if(!viewer_)
     return;
  if( get_mode1() )
     effect_->mode( get_mode1()|get_mode2()|PLAY_ON );
  else
     effect_->mode( 0x0000L );
}


void EffectPlayer::accept1(FieldEditor*)
{
  if(!viewer_)
    return;
  effect_->start(atoi(mstart_->text()->string()));
  time();
}

void EffectPlayer::cancel1(FieldEditor*){}

void EffectPlayer::accept2(FieldEditor*)
{
  if(!viewer_)
    return;
  effect_->end(atoi(mend_->text()->string()));
  time();
}

void EffectPlayer::cancel2(FieldEditor*){}

void EffectPlayer::accept3(FieldEditor*)
{
  if(!viewer_)
    return;
  effect_->difference(atoi(mdiffer_->text()->string()));
  time();
}

void EffectPlayer::cancel3(FieldEditor*){}

unsigned long EffectPlayer::get_mode1()
{
  int i;
  for( i = 0;  i < 9 ; i++ )
    if(buttons1_[i]->state()->flags() & 0x08)
      break;
  if( i == 0 )
    return MOTION_WIPE;
  else if( i == 1 )
    return MOTION_SLIDE;
  else if( i == 2 )
    return MASK_IRISSQUARE;
  else if( i == 3 )
    return MASK_IRISROUND;
  else if( i == 4 )
    return MASK_LINEARBOXES;
  else if( i == 5 )
    return MASK_SPIRALBOXES; /* ADD 1994/08/12 */
  else if( i == 6 )
    return MASK_DISSOLVE;
  else if( i == 7 )
    return MASK_ZOOM;
  else if( i == 8 )
    return MOTION_STRETCH;
  else
    return 0x0000;
}

unsigned long EffectPlayer::get_mode2()
{
  unsigned long mode = 0x0000L;
  if( buttons2_[0]->state()->flags() & 0x08)
    mode = PLAY_REVERSE;
  if( buttons2_[1]->state()->flags() & 0x08)
    mode |= PLAY_DIMENSION;
  return mode;
}

void EffectPlayer::change_media()
{
   WidgetKit& kit    = *WidgetKit::instance();
   LayoutKit& layout = *LayoutKit::instance();
   char message[400];

   if( window_ ){
      if(window_->is_mapped()){
	 viewer_->stop();
	 if(window_ != nil){
	    HsDefaultColormap::uninstall(window_);
	    window_->unmap();
	 }
      }
      delete window_;
   }
   
   HsTime diff, start, end;
   
   diff  = atoi(mdiffer_->text()->string());
   start = atoi(mstart_->text()->string());
   end   = atoi(mend_->text()->string());
   
   type1_ = type_check( nfield1_->text()->string() );
   type2_ = type_check( nfield2_->text()->string() );

   if( type1_ == NON && type2_ == NON ){
     sprintf( message, "Could not find file %s and %s. ", 
	      nfield1_->text()->string(), nfield2_->text()->string() );
     show_message(message);
     return;
   }else if( type1_ == NON ){
     sprintf( message, "Could not find file %s. ", 
	      nfield1_->text()->string() );
     show_message(message);
     return;
   }else if( type2_ == NON ){
     sprintf( message, "Could not find file %s. ", 
	      nfield2_->text()->string() );
     show_message(message);
     return;
   }
   Display* dpy = HsSession::instance()->default_display();

   if( ( ( type1_ != MPEG && type1_ != JPEG ) || 
	 ( type2_ != MPEG && type2_ != JPEG ) )
      && dpy->rep()->default_visual_->depth() != 8 ){
     show_message("This data is not a full color data.");
     return;
   }

   sprintf( message,"Load data file %s and %s.",
	    nfield1_->text()->string(),
	    nfield2_->text()->string() );
   show_message(message);

   HsMedia* media1 = type_for_media( type1_, nfield1_->text()->string() );
   HsMedia* media2 = type_for_media( type2_, nfield2_->text()->string() );
   
   if( filename1_ )
      delete[] filename1_;
   if( filename2_ )
      delete[] filename2_;

   filename1_ = new char[strlen(nfield1_->text()->string())+1];
   strcpy(filename1_,nfield1_->text()->string());
   filename2_ = new char[strlen(nfield2_->text()->string())+1];
   strcpy(filename2_,nfield2_->text()->string());

   effect_ = new HsEffect( media1, media2, diff, start, end );
   viewer_ = new HsViewer( effect_ );
   Glyph* viewbody = 
      layout.vbox( kit.outset_frame( viewer_ ),
		   new HsViewerGadget( viewer_ ) );
   
   window_ =
      new TransientWindow( kit.outset_frame( viewbody ) );
   update();
   time();
   map();
}

void EffectPlayer::choose(FileChooser*,boolean flag)
{
  if( flag == true)
    select_media();
}

void EffectPlayer::select_media()
{
  if(buttons3_[0]->state()->flags() & 0x08){
    nfield1_->field(fchooser_->selected()->string());
  }else if(buttons3_[1]->state()->flags() & 0x08){
    nfield2_->field(fchooser_->selected()->string());
  }
}

void EffectPlayer::time()
{
  char buff[40];
  sprintf( buff,"%4.0f  ", viewer_->media()->duration() );
  duration1_->change_label( buff );
  sprintf( buff,"%4.0f  ", viewer_->media()->component(0)->duration() );
  duration2_->change_label( buff );
  sprintf( buff,"%4.0f  ", viewer_->media()->component(1)->duration() );
  duration3_->change_label( buff );
  sprintf(buff,"%.0f", effect_->difference());
  mdiffer_->field(buff);
  sprintf(buff,"%.0f", effect_->start());
  mstart_->field(buff);
  sprintf(buff,"%.0f", effect_->end());
  mend_->field(buff);
}

Glyph* EffectPlayer::make_margin( Glyph* glyph )
{
   WidgetKit& kit    = *WidgetKit::instance();
   LayoutKit& layout = *LayoutKit::instance();
   
   Glyph* g = 
     kit.outset_frame(
	   layout.margin(
            kit.inset_frame( glyph ), 5
			 )
		      );
   return g;
}
        
void EffectPlayer::show_message(char* message)
{
  WidgetKit& kit    = *WidgetKit::instance();
  char buff[24];
  int len = strlen(message);
  char* cp = message;

  while( 2 < box_->count() )
    box_->remove(0);

  kit.push_style( new Style( kit.style() ) );
  kit.style()->attribute( "font", 
                          "*-courier-medium-r-normal--18-180-75-75-*" );
  while( 0 < len ){
    strncpy( buff, cp, 20 );
    if(20 < len)
      buff[20]='\0';
    else
      buff[len]='\0';
    len -= 20;
    box_->insert( box_->count()-2, kit.label(buff) );
    cp += 20;
  }
  kit.pop_style();
  
  patch_->reallocate();
  patch_->redraw();
}

HsMedia* EffectPlayer::type_for_media(DataType type, const char* name)
{
  HsMedia* media = nil;
  switch(type)
    {
    case AVFILE:
      media = HsAVdesc::open( name );
      break;
    case COLOR:
      media = new HsDigitizedVideo( name );
      break;
    case MONO:
      media = new HsDigitizedVideo( name );
      break;
    case MPEG:
      media = new HsMPEGVideo( name );
      break;
    case JPEG:
      media = new HsJPEGVideo( name );
      break;
    case NON:
      break;
    }
  return media;
}

void EffectPlayer::avdump()
{
   char obj1[20];
   char obj2[20];
   long id;

   struct timeval  tv;

   if( !filename1_ || !filename2_ )
      return;

   if( type1_ == NON || type2_ == NON )
      return;
   
   FILE* avfp;
   if( avfile_ ){
      if((avfp=fopen( filename1_,"w" )) == NULL){
	 fprintf( stderr, "Could not open file %s\n", avfile_);
	 return;
      }
   }else{
      if((avfp=fopen( AVOUTFILE,"w" )) == NULL){
	 fprintf( stderr, "Could not open file %s\n", avfile_);
	 return;
      }
   }

   fprintf( avfp, "%%!HYPER-VIDEO 0.1\n" );
   avlist( type1_, avfp, effect_->component(0), filename1_, obj1, false );
   avlist( type2_, avfp, effect_->component(1), filename2_, obj2, true );

#ifdef nec_ews
    gettimeofday(&tv);
#else    
    struct timezone tzp2;
    gettimeofday(&tv, &tzp2);
#endif

   idmake(id);
   fprintf( avfp, "mix%ld:\n\tEffect %s %s %s %s %s ",
	    id, obj1, obj2, mdiffer_->text()->string(),
	    mstart_->text()->string(), mend_->text()->string() );

   if( effect_->mode() & PLAY_ON ){
      if(effect_->mode() & (MOTION_SLIDE) ){
	 if( effect_->mode() &  PLAY_REVERSE ){
	    if( effect_->mode() & PLAY_DIMENSION )
	       fprintf( avfp, "SLIDE_RV\n");
	    else
	       fprintf( avfp, "SLIDE_RH\n");
	 }else{
	    if( effect_->mode() & PLAY_DIMENSION )
	       fprintf( avfp,"SLIDE_V\n");
	    else
	       fprintf( avfp,"SLIDE_H\n");
	 }
      }else if(effect_->mode() & (MOTION_WIPE) ){
	 if( effect_->mode() &  PLAY_REVERSE ){
	    if( effect_->mode() & PLAY_DIMENSION )
	       fprintf( avfp, "WIPE_RV\n");
	    else
	       fprintf( avfp, "WIPE_RH\n");
	 }else{
	    if( effect_->mode() & PLAY_DIMENSION )
	       fprintf( avfp,"WIPE_V\n");
	    else
	       fprintf( avfp,"WIPE_H\n");
	 }
      }else if(effect_->mode() & (MOTION_STRETCH) ){
	 if( effect_->mode() &  PLAY_REVERSE ){
	    if( effect_->mode() & PLAY_DIMENSION )
	       fprintf( avfp, "STRETCH_RV\n");
	    else
	       fprintf( avfp, "STRETCH_RH\n");
	 }else{
	    if( effect_->mode() & PLAY_DIMENSION )
	       fprintf( avfp,"STRETCH_V\n");
	    else
	       fprintf( avfp,"STRETCH_H\n");
	 }
      }else if(effect_->mode() & MASK_ZOOM )
	 fprintf( avfp, "ZOOM\n");
      else if(effect_->mode() & MASK_DISSOLVE )
	 fprintf( avfp, "DISSOLVE\n");
      else if(effect_->mode() & MASK_IRISSQUARE )
	 fprintf( avfp, "IRISSQUARE\n");
      else if(effect_->mode() & MASK_IRISROUND )
	 fprintf( avfp, "IRISROUND\n");
      else if(effect_->mode() & MASK_LINEARBOXES )
	 fprintf( avfp, "LINEARBOXES");
      else if(effect_->mode() & MASK_SPIRALBOXES )
	 fprintf( avfp, "SPIRALBOXES");
   } else
      fprintf( avfp, "NORMAL\n");
   
   fclose(avfp);
}

void get_line(char* line) {
   char buf[1000];
   strcpy(buf, line);
   char *ptr;
   for(ptr = buf + strlen(buf) - 1; buf <= ptr && isspace(*ptr); ptr--)
      *ptr = '\0';
   for (ptr = buf; *ptr && isspace(*ptr); ptr++)
      ;
   strcpy(line, ptr);
}


void EffectPlayer::avlist( DataType type, FILE* avfp, HsMedia* media,
			   char* filename, char* obj, boolean flag )
{
   char buff[1000];
   int cnt = 0;
   long id=0;

   for( int i = 0; i < idcnt_; i++ ){
      delete[] oldnamelist_[i];
      delete[] newnamelist_[i];
   }
   oldnamelist_[0] = nil;
   newnamelist_[0] = nil;
   idcnt_       =   0;

   if( type == AVFILE ){
      FILE* fp;
      if((fp=fopen( filename,"r" )) == NULL){
	 fprintf( stderr, "Could not open file %s\n", filename);
	 return;
      }
      if(flag)
	 fgets( buff, 240, fp );
      while( fgets( buff, 800, fp ) != NULL ){
	 char* ptr;
	for(ptr = buff + strlen(buff) - 1; buff <= ptr && isspace(*ptr); ptr--)
	   *ptr = '\0';
	 if( buff[strlen(buff)-1] == ':' ){
	    buff[strlen(buff)-1] = '\0';
	    setid(buff);
	    char* cname = checkid(buff);
	    fprintf(avfp, "%s:\n",cname);
	    strcpy(obj,cname);
	 }else if( buff[0] != '%' ){
	    char classname[40];
	    char* cname;
	    sscanf(buff,"%s", classname );
	    if(!strcmp("Cut", classname)){
	       fprintf(avfp, "%s\n",buff);
	       if(fgets( buff, 800, fp ) == NULL)
		  break;
	       sscanf(buff,"%s", classname);
	       cname = checkid(classname);
	       fprintf(avfp, "\t%s\n",cname);
	    }else if(!strcmp("VBox", classname)){
	       fprintf(avfp,"\tVBox\n");
	       while( fgets( buff, 800, fp ) != NULL ){
		  sscanf(buff,"%s", classname );
		  if(!strcmp("END", classname)){
		     fprintf(avfp, "\tEND\n",cname);
		     break;
		  }
		  cname = checkid(classname);
		  fprintf(avfp, "\t%s\n",cname);
	       }
	    }else if(!strcmp("RVBox", classname)){
	       fprintf(avfp,"\tRVBox\n");
	       while( fgets( buff, 800, fp ) != NULL ){
		  sscanf(buff,"%s", classname );
		  if(!strcmp("END", classname)){
		     fprintf(avfp, "\tEND\n",cname);
		     break;
		  }
		  cname = checkid(classname);
		  fprintf(avfp, "\t%s\n",cname);
	       }
	    }else if(!strcmp("HBox", classname)){
	       fprintf(avfp,"\tHBox\n");
	       while( fgets( buff, 800, fp ) != NULL ){
		  sscanf(buff,"%s", classname );
		  if(!strcmp("END", classname)){
		     fprintf(avfp, "\tEND\n",cname);
		     break;
		  }
		  cname = checkid(classname);
		  fprintf(avfp, "\t%s\n",cname);
	       }
	    }else if(!strcmp("TBox", classname)){
	       fprintf(avfp,"\tTBox\n");
	       while( fgets( buff, 800, fp ) != NULL ){
		  sscanf(buff,"%s", classname );
		  if(!strcmp("END", classname)){
		     fprintf(avfp, "\tEND\n",cname);
		     break;
		  }
		  cname = checkid(classname);
		  fprintf(avfp, "\t%s\n",cname);
	       }
	    }else if(!strcmp("Overlay", classname)){
	       fprintf(avfp,"\tOverlay\n");
	       while( fgets( buff, 800, fp ) != NULL ){
		  sscanf(buff,"%s", classname );
		  if(!strcmp("END", classname)){
		     fprintf(avfp, "\tEND\n",cname);
		     break;
		  }
		  cname = checkid(classname);
		  fprintf(avfp, "\t%s\n",cname);
	       }
	    }else if(!strcmp("Effect", classname)){
	       int diff, start, end;
	       char filename1[120], filename2[120], etype[24];
	       char* ptr = buff;
	       while( ptr[0] == ' '|| ptr[0] == '\t' )
		  ptr++;
	       sscanf(ptr,"Effect %s %s %d %d %d %s",
		      filename1, filename2, &diff, &start, &end, etype );
	       fprintf(avfp,"\tEffect %s %s %d %d %d %s\n",
		       checkid(filename1), checkid(filename2), 
		       diff, start, end, etype );
	    }else if(!strcmp("Title", classname)){
	       int dx,dy,dz,sx,sy,sz;
	       char bkname[200], font[200], color[40], tmode[24];
	       char* ptr = buff;
	       while( ptr[0] == ' '|| ptr[0] == '\t' )
		  ptr++;
	       sscanf(ptr,"Title %s %d %d %d %d %d %d %s %s %s",
		      bkname,&dx,&dy,&dz,&sx,&sy,&sz,font,color,tmode );
	       fprintf(avfp,"\tTitle %s %d %d %d %d %d %d %s %s %s\n",
		       checkid(bkname),dx,dy,dz,sx,sy,sz,font,color,tmode );
	       if( fgets( buff, 800, fp ) == NULL )
		  break;
	       fprintf( avfp,"%s", buff );

	    }else
	       fprintf( avfp,"%s\n", buff );
	 }
      }
      fclose(fp);
   }
   
   if( type == COLOR ){
      HsDigitizedVideo* dv = (HsDigitizedVideo*)media;
      setid("color");
      strcpy(obj, checkid("color"));
      fprintf( avfp, "%s:\n\tDigitizedVideo %s %d %d %d\n",
	       obj, filename, dv->pwidth(), dv->pheight(), dv->depth());
   }else if( type == MONO ){
      HsDigitizedVideo* dv = (HsDigitizedVideo*)media;
      setid("mono");
      strcpy(obj, checkid("mono"));
      fprintf( avfp, "%s:\n\tDigitizedVideo %s %d %d %d\n",
	       obj, filename, dv->pwidth(), dv->pheight(), dv->depth());
   }else if( type == MPEG ){
      HsMPEGVideo* mp = (HsMPEGVideo*)media;
      setid("mpeg");
      strcpy(obj, checkid("mpeg"));
      fprintf( avfp, "%s:\n\tMPEGVideo %s %d %d %.0f\n",
	       obj, filename, mp->pwidth(), mp->pheight(), mp->duration());
   }else if( type == JPEG ){
      HsJPEGVideo* jp = (HsJPEGVideo*)media;
      setid("jpeg");
      strcpy(obj, checkid("jpeg"));
      fprintf( avfp, "%s:\n\tJPEGVideo %s %d %d %.0f\n",
	       obj, filename, jp->pwidth(), jp->pheight(), jp->duration());
   }
   for( int l = 0; l < idcnt_; l++ )
      fprintf(stderr,"%d >%s< >%s<\n", l, oldnamelist_[l], newnamelist_[l] );
}

void EffectPlayer::idmake( long& id )
{
   struct timeval  tv;
#ifdef nec_ews
   gettimeofday(&tv);
#else    
   struct timezone tzp2;
   gettimeofday(&tv, &tzp2);
#endif
   id = tv.tv_usec/10000+(tv.tv_sec%10000)*100;
   while( id_ == id ){
#ifdef nec_ews
      gettimeofday(&tv);
#else    
      gettimeofday(&tv, &tzp2);
#endif
      id = tv.tv_usec/10000+(tv.tv_sec%10000)*100;
   }
   id_ = id;
}

void EffectPlayer::setid(char* name)
{
   char buf[8];
   long id;
   
   oldnamelist_[idcnt_] = new char[strlen(name)+1];
   strcpy(oldnamelist_[idcnt_], name);
   strncpy(buf,name,4);
   buf[strlen(buf)-1] = '\0';
   newnamelist_[idcnt_] = new char[12];
   idmake(id);
   sprintf(newnamelist_[idcnt_],"%s%ld", buf, id );
   idcnt_++;
   oldnamelist_[idcnt_] = nil;
   newnamelist_[idcnt_] = nil;
}

char* EffectPlayer::checkid(char* oldname)
{
   for( int i = 0; i < idcnt_; i++ )
      if(!strcmp(oldname,oldnamelist_[i]))
	 return newnamelist_[i];
   return nil;
}

