// -*-c++-*-
// Copyright (C) 1995 NEC Corporation, Tokyo, Japan.

#include <InterViews/background.h>
#include <InterViews/window.h>
#include <InterViews/display.h>
#include <IV-look/kit.h>
#include <IV-look/fbrowser.h>
#include <InterViews/target.h>
#include <InterViews/session.h>
#include <IV-X11/xdisplay.h>
#include <Hs/HsSession.h>
#include "MultiMenu.h"
#include "MultiPlayer.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern DataType type_check( const char* name );

static PropertyData props[] = {
    { "title",  "MultiPlayer" },
    { nil }
};

static void error_message()
{
   fprintf( stderr, "Usage: player [ [-nolabel] [-nospeed] ] [-nomenu] [filename]\n");
   exit(0);
}

boolean _nolabel = false;
boolean _nospeed = false;

int main(int argc, char** argv) {

   HsSession* session = new HsSession( "MultiPlayer", argc, argv, nil, props );
   WidgetKit* kit = WidgetKit::instance();
   MultiMenu* mmenu = nil;
   MultiPlayer* player;
   DataType type = NON;
   int cnt = argc-1;
   char typename[8];
   boolean nomenu = false;

   if( 5 < argc ){
      error_message();
      exit(0);
   }
   if( cnt ){
      if( !strcmp( argv[argc-cnt], "-nolabel" ) ){
         _nolabel = true;
         cnt--;
         if( 0 == cnt )
            error_message();
      }

      if( !strcmp( argv[argc-cnt], "-nospeed" ) ){
         _nospeed = true;
         cnt--;
         if( 0 == cnt )
	    error_message();
      }

      if( !strcmp( argv[argc-cnt],"-nomenu") ){
         nomenu = true;
         cnt--;
         if( 0 == cnt )
	    error_message();
      }
   }
   if( 0 < cnt  ){
      if( argv[cnt-1][0] != '-' ){
	 type = type_check(argv[argc-cnt]);
	 if( type == NON ){
	    fprintf( stderr,"Can't find file %s\n", argv[argc-cnt] );
	    exit(0);
	 }
	 cnt--;
      }else{
	 error_message();
      }
   }
   if( 0 != cnt )
      error_message();
   if( type != NON ){
      int depth = 
	 HsSession::instance()->default_display()->rep()->default_visual_->depth();
      if( depth != 8 ){
	 if( type == MONO || type == COLOR ){
	    fprintf( stderr, "This data, %s is not a full color data.\n",
		     argv[argc-1] );
	    exit(0);
	 }else if( type == AVFILE ){
	    FILE* fp;
	    if( ( fp = fopen( argv[argc-1],"r" ) ) == NULL ){
	       fprintf( stderr,"Can't open file %s\n", argv[argc-1] );
	       exit(0);
	    }
	    char name1[120], name2[120], buff[400];
	    int w,h,d;
	    while( fgets( buff, 160, fp ) ){
	       sscanf(buff,"%s %s %d %d %d",name1, name2, &w, &h, &d );
	       if(!strcmp( "DigitizedVideo", name1 )){
		  fprintf( stderr, "This data, %s is not a full color data.\n",
			   argv[argc-1] );
		  exit(0);

	       }
	    }
	    fclose(fp);
	 }
      }
      if(!nomenu)
	 mmenu = new MultiMenu;
      switch (type){
      case AVFILE:
	 strcpy(typename,"AVFILE");
	 break;
      case COLOR:
	 strcpy(typename,"COLOR");
	 break;
      case MONO:
	 strcpy(typename,"MONO");
	 break;
      case MPEG:
	 strcpy(typename,"MPEG");
	 break;
      case JPEG:
	 strcpy(typename,"JPEG");
	 break;
      case NON:
	 strcpy(typename,"NON");
	 break;
      }
	 
      fprintf(stderr, "I assume that %s is %s format.\n", argv[argc-1], typename );

      player = new MultiPlayer( type, argv[argc-1], mmenu );
      player->map();
   }else{
      mmenu = new MultiMenu();
      mmenu->map();
   }
   
   session->run();
   delete session;

   return(0);
}
