// -*-c++-*-
// Copyright (C) 1992, 1993, 1994, 1995 NEC Corporation, Tokyo, Japan.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <InterViews/background.h>
#include <InterViews/resource.h>
#include <InterViews/box.h>
#include <InterViews/color.h>
#include <InterViews/layout.h>
#include <InterViews/style.h>
#include <InterViews/session.h>
#include <InterViews/window.h>
#include <InterViews/font.h>
#include <IV-look/kit.h>

#include <Hs/HsViewer.h>
#include <Hs/HsDigitizedVideo.h>
#include <Hs/HsMPEGVideo.h>
#include <Hs/HsJPEGVideo.h>
#include <Hs/HsSession.h>
#include <Hs/HsAVdesc.h>
#include <Hs/HsDefaultColormap.h>

typedef enum {
  AVFILE,
  COLOR,
  MONO,
  MPEG,
  JPEG,
  NON,
}DataType;

static PropertyData props[] = {
    { "*transient", "on" },
    { nil }
};

static OptionDesc options[] = {
    { "-top", "*transient", OptionValueImplicit, "off" },
    { nil }
};

static char _mpegformat[8]  = { 0x00, 0x00, 0x01, 0xb3 };
static char _jpegformat[8] = { 0xff, 0xd8, 0xff, 0xe0 };

boolean checking_mpeg(char* buff)
{
  int i;
  for( i = 0; i < 4; i++)
    if( buff[i] != _mpegformat[i] )
      break;
  if( 4 == i )
    return true;
  return false;
}

boolean checking_jpeg(char* buff)
{
  int i;
  for( i = 0; i < 4; i++)
    if( buff[i] != _jpegformat[i] )
      break;
  if( 4 == i )
    return true;
  return false;
}

DataType type_check( const char* n )
{
  char buff[256];
  char state[40];
  char* name;
  char* cp;
  int l = 0;
  DataType type;
  
  FILE* fp;
  int len = strlen(n);
  name = new char[ len+1 ];
  strcpy( name , n );

  if( name[len-1] == '/'){
    delete[] name;
    return NON;
  }
  if( (fp = fopen( name, "r" ) ) == NULL ){
    fprintf(stderr,"Could not open File %s\n", n );
    delete[] name;
    return NON;
  }
  fgets( buff, 80, fp );

  cp = buff;
  if(!strncmp("%!HYPER-VIDEO", buff, 13 )){
     delete[] name;
     fclose(fp);
     return AVFILE;
  }

  int cnt = 0;

  if( !strncmp("RAWMOVIE ", buff, 9 )){
    while(cp[0] != '\0'){
      if( cp[0] != ' ' && cp[1] == ' ' )
	cnt++;
      if( cnt == 3 && cp[0] == ' ' && cp [1] != ' ' && cp[1] != '\0'){
	sscanf(cp,"%s", state);
	l = atoi(state);
      }
      cp++;
    }
  }
  if(l == 1)
    type = MONO;
  else if(l == 8)
    type = COLOR;
  else{
     cp = name;
     int i;
     for( i = strlen( name )-1; 0 <= i; i-- )
	if( name[i] == '/' ){
	   cp = &name[i];
	   break;
	}
     for ( l = strlen( cp ) -1; cp[l] != '.'&& 0 <= l; l-- )
	;
     cp += l;

     if( checking_mpeg( buff ) )
        type = MPEG;
     else if( checking_jpeg( buff ) )
        type = JPEG;
     else{
	printf("This is not supported data \"%s\".\n", name);        
	type = NON;
     }
  }
  
  delete[] name;
  fclose(fp);
  return type;
}

int main(int argc, char** argv) {
    HsSession *session = new HsSession("Player", argc, argv, options, props);
    Display* d = session->default_display();
    Style* style = session->style();
    WidgetKit* kit = WidgetKit::instance();
    LayoutKit* layout = LayoutKit::instance();

    char* name = nil;

    if (argc <= 1) {
	printf("%s: usage:\n\t%s file \n", argv[0], argv[0]);
	return 0;
    }

    if (argc > 1) name = argv[1];
    
    FILE *f = fopen(name, "r");
    if (f == nil) {
	fprintf(stderr, "%s: can't open %s\n", argv[0], name);
	return 0;
    }
    
    DataType dtype = type_check(name);
    
    HsMedia *media;

    switch(dtype) {
        case AVFILE:
            media = HsAVdesc::open(name);
	    break;
	case COLOR:
	    media = new HsDigitizedVideo(name);
	    break;
        case MONO:
	    media = new HsDigitizedVideo(name);
	    break;
	case MPEG:
	    media = new HsMPEGVideo(name);
	    break;
	case JPEG:
	    media = new HsJPEGVideo(name);
	    break;
	case NON:
	    return 0;
    }

    HsViewer* viewer = new HsViewer();
    viewer->media(media);

    Glyph* dialog = new Background(
	layout->vbox(viewer, new HsViewerGadget(viewer)),
	kit->background()
    );

    Window* w = new ApplicationWindow(dialog);

    w->align(0.5, 0.5);
    w->map();
    HsDefaultColormap::install(w);
    session->run();
    delete w;
    delete session;
    return 0;
}


    
    

